
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#ifndef _BACKGROUNDPAINTER_H_
#define _BACKGROUNDPAINTER_H_

#include <QtCore/QObject>
#include <QtGui/QImage>
#include <QtGui/QColor>

class BackgroundPainter :
    public QObject
{
    Q_OBJECT

public:

    enum MultiMonitorSupportType
    {
        MULTIMONITOR_NO_MULTIMONITOR = 0,
        MULTIMONITOR_ONE_IMAGE_MULTIPLEMONITOR,
        MULTIMONITOR_ONE_IMAGE_BY_MONITOR,
        MULTIMONITOR_END
    };

    enum ResizingType
    {
        SCALE_NO_RESIZE = 0,
        SCALE_IGNORE_ASPECT,
        SCALE_KEEP_ASPECT,
        SCALE_KEEP_EXPANDED_ASPECT,
        SCALE_END
    };

    enum VerticalAlignmentType
    {
        ALIGN_V_TOP = 0,
        ALIGN_V_CENTER,
        ALIGN_V_BOTTOM,
        ALIGN_V_USER,
        ALIGN_V_END
    };

    enum HorizontalAlignmentType
    {
        ALIGN_H_LEFT = 0,
        ALIGN_H_CENTER,
        ALIGN_H_RIGHT,
        ALIGN_H_USER,
        ALIGN_H_END
    };

    enum FillingType
    {
        FILL_BY_COLOR = 0,
        FILL_BY_SMART_COLOR,
        FILL_BY_BLUR,
        FILL_BY_VERTICAL_GRADIENT,
        FILL_BY_HORIZONTAL_GRADIENT,
        FILL_BY_RADIAL_GRADIENT,
        FILL_END
    };

    enum SpecialEffectType
    {
        EFFECT_NONE = 0,
        EFFECT_GRAYSCALE,
        EFFECT_BLUR,
        EFFECT_GAUSSIAN_BLUR,
        EFFECT_EDGE,
        EFFECT_SHARPEN,
        EFFECT_GAUSSIAN_SHARPEN,
        EFFECT_EMBOSS,
        EFFECT_DESATURATE,
        EFFECT_FLATTEN,
        EFFECT_SWIRL,
        EFFECT_OILPAINT,
        EFFECT_CHARCOAL,
        EFFECT_SEPIA,
        EFFECT_INVERTCOLOR,
        EFFECT_GLOW,
        EFFECT_PROGRESSIVEBLUR,
        EFFECT_RANDOM,
        EFFECT_END
    };

    enum RotationModeType
    {
        ROTATION_UNLNOWN = 0,
        ROTATION_NONE,
        ROTATION_FLIP_HORIZ,
        ROTATION_180,
        ROTATION_FLIP_VERTICAL,
        ROTATION_TRANSPOSE,
        ROTATION_90,
        ROTATION_TRANSVERSE,
        ROTATION_270,
        ROTATION_END
    };

    enum SmoothScaleType
    {
        SMOOTH_DEFAULT = 0,
        SMOOTH_BLITZ_BLACKMANFILTER,
        SMOOTH_BLITZ_CUBICFILTER,
        SMOOTH_BLITZ_MITCHELLFILTER,
        SMOOTH_BLITZ_LANCZOSFILTER
    };

    BackgroundPainter(QObject *parent = NULL);
    virtual ~BackgroundPainter();

    void setMultiMonitorSupportType(int);
    int getMultiMonitorSupportType() const
    {
        return m_multiMonitorType;
    }

    void setResizingType(int);
    int getResizingType() const
    {
        return m_resizingType;
    }

    void setVerticalAlignmentType(int);
    int getVerticalAlignmentType() const
    {
        return m_verticalAlignmentType;
    }

    void setHorizontalAlignmentType(int);
    int getHorizontalAlignmentType() const
    {
        return m_horizontalAlignmentType;
    }

    void setFillingType(int);
    int getFillingType() const
    {
        return m_fillingType;
    }

    void setSpecialEffectType(int);
    int getSpecialEffectType() const
    {
        return m_specialEffectType;
    }

    int getLastSpecialEffectType() const
    {
        return m_lastSpecialEffectType;
    }

    void setSmoothAlgorithm(int);
    int getSmoothAlgorithm() const
    {
        return m_smoothAlgorithm;
    }

    void setSpecialEffectParameter(float val);
    float getSpecialEffectParameter() const { return m_specialEffectParam; }

    void setShareImageBetweenScreen(bool);
    bool getShareImageBetweenScreen() const
    {
        return m_shareImageBetweenScreen;
    }

    void setFillingColor1(const QColor &color);
    const QColor &getFillingColor1() const {return m_fillingColor1; }

    void setFillingColor2(const QColor &color);
    const QColor &getFillingColor2() const {return m_fillingColor2; }

    void setInverseGradient(bool);
    bool getInverseGradient() const
    {
        return m_inverseGradient;
    }

    QImage processImage(const QImage &srcImage, bool previewMode = false);

    void setTargetSizeWithMargins(const QSize &size){m_targetSizeWithMargins = size; }
    QSize getTargetSizeWithMargins() const { return m_targetSizeWithMargins; }

    void setTargetSize(const QSize &size){m_targetSize = size; }
    QSize getTargetSize() const { return m_targetSize; }

    QSize getSizeAccordingToConfig(const QSize &baseSize, bool preview = false);

    void fillImage(const QImage &src, QImage &target);
    void postProcess(QImage &, int effect, bool preview = false);

    QImage cropAndMerge(const QImage & background,const QList<QRectF> &listGamingArea);
    QList<QImage> cropAndSplit(const QImage & background,const QList<QRectF> &listGamingArea);
    QImage crop(const QImage & background,const QSize &size, bool previewMode = false);
    void embedImage(const QImage&src, QImage & desc);

    bool hasSpecialEffectParameter(int) const;
    QImage transform(const QImage &image, int mode) const;
    QImage smoothScale(const QImage &image,const QSize &newSize,int mode) const;

    void setUserOffset(const QPointF &);
    const QPointF & getUserOffset() const
    {
        return m_userOffset;
    }

public slots:

    void saveSettings();

signals:

private:

    void loadSettings();

    QColor computeSmartColor(const QImage &img);
    QImage toSepia(const QImage &img);
    QImage invertColors(const QImage &img);
    QImage glow(const QImage &image, int radius);
    QImage progressiveBlur(const QImage &image, int radius);

    bool m_shareImageBetweenScreen;
    int m_multiMonitorType;
    int m_resizingType;
    int m_verticalAlignmentType;
    int m_horizontalAlignmentType;
    int m_fillingType;
    int m_specialEffectType;
    int m_lastSpecialEffectType;
    float m_specialEffectParam;

    QColor m_fillingColor1;
    QColor m_fillingColor2;
    bool m_inverseGradient;

    QSize m_targetSize;
    QSize m_targetSizeWithMargins;

    QList<int> m_hasParameter;
    int m_smoothAlgorithm;
    bool m_sharpenSmoothScale;

    QPointF m_userOffset;
};

#endif // _BACKGROUNDPAINTER_H_
