
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
#ifndef _BACKGROUNDPAINTERCONFIGWIDGET_H_
#define _BACKGROUNDPAINTERCONFIGWIDGET_H_

#include <QtGui/QWidget>

#include "BackgroundPainter.h"

// forward declarations
class DesktopScene;
class ScreenItem;
class NetworkMessage;

class QPushButton;

namespace Ui
{
class BackgroundPainterConfigWidget;
}

class BackgroundPainterConfigWidget :   public QWidget
{
    Q_OBJECT

public:

    explicit BackgroundPainterConfigWidget(QWidget *parent = 0);
    ~BackgroundPainterConfigWidget();

    DesktopScene *getScenePreview() const { return m_desktopScene; }
    bool isUserMoveAllowed() const;

    void processImage(const QString &,bool manual = false, bool force = false);

    //! valid the current position of the image (user defined)
    QPointF validCurrentUserPositioning();

    //! get the number of screen take into account by current configuration
    int getScreenNumberByConfig() const;

    void setUserOffset(const QPoint &);

public slots:

    void decodeAndApplyMessage(const QString &);
    void decodeAndApplyMessage(const NetworkMessage &);

    void saveSettings();

    void updateScreenConfig(int);

    void previewImage( const QString &file = QString() );

signals:

    //! request a new image to display according to user parameter (current one don't match)
    void requestNewImage();

    //! send computed image(s) to display
    void newImage(const QImage &,bool);
    void newImages(const QList<QImage> &,bool);

    //! send new configuration string to client
    void newConfiguration(const QString &);
    void newConfiguration(const NetworkMessage &);

    //! send decoded image from server string to display
    void decodedImage(const QString &);

    //! send that user change parameters
    void parametersChange();

    void newNetworkImage(const QString &);

private slots:

    void on_checkBoxShareScreen_stateChanged ( int state );
    void on_checkBoxInverseGradient_stateChanged ( int state );
    void on_checkBoxIgnoreParameters_stateChanged ( int state );
    void on_checkBoxExportParameters_stateChanged ( int state );
    void on_comboBoxResizingType_activated(int index);
    void on_comboBoxAlignmentTypeVertical_activated(int index);
    void on_comboBoxAlignmentTypeHorizontal_activated(int index);
    void on_comboBoxFillingType_activated(int index);
    void on_comboBoxSpecialEffect_activated(int index);
    void on_pushButtonPreviewColor1_pressed();
    void on_pushButtonPreviewColor2_pressed();
    void on_doubleSpinBoxSpecialEffectParam_valueChanged(double);
    void on_comboBoxMultiMonitor_activated(int);
    void on_checkBoxSkipTooSmallImage_stateChanged ( int state );
    void on_checkBoxRotateWithMetadata_stateChanged ( int state );
    void on_comboBoxSamplingMethod_activated(int index);
    void on_checkBoxKeepWMSettings_stateChanged ( int state );

private:

    void loadSettings();

    void updateUI();
    void applyConfig();
    void createDesktop();
    void updateDesktop();
    void initResizingType();
    void initAlignmentType();
    void initMultiMonitorSupportType();
    void initFillingType();
    void initSamplingMethod();
    void initSpecialEffectType();
    void setButtonColor(const QColor &c, QPushButton *b);
    QString encodeConfiguration(const QString &);

    Ui::BackgroundPainterConfigWidget *ui;

    BackgroundPainter m_painter;
    DesktopScene *m_desktopScene;
    QImage m_previewImage;
    QString m_previewFile;

    QList<ScreenItem *> m_screens;
    bool m_exportParameters;
    bool m_ignoreParameters;
    bool m_skipTooSmallImage;
    bool m_rotateFromMetadata;
    bool m_keepWMSettings;
    QString m_lastFile;
};

#endif // _BACKGROUNDPAINTERCONFIGWIDGET_H_
