
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include "DesktopView.h"
#include "DesktopScene.h"

//#include "ScreenItem.h"

#include <QtCore/QDebug>
#include <QtGui/QApplication>
#include <QtGui/QPalette>

//#include <QtOpenGL/QGLWidget>

DesktopView::DesktopView(QWidget * parent)
    : QGraphicsView(parent)
{
    // a gradient background
    //QLinearGradient lgradient(0, 0,1000,1000); // here 1000x1000 pixel !!
    QRadialGradient lgradient(0.0,0.0,2000);
    lgradient.setSpread(QGradient::PadSpread);
    lgradient.setColorAt( 1, QColor("#4a4a4a") );
    lgradient.setColorAt( 0, QColor("#797979") );
    setBackgroundBrush(lgradient);

    setCacheMode( QGraphicsView::CacheBackground );

    setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
    setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform);

    setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    setFrameStyle( QFrame::NoFrame );
}

DesktopView::~DesktopView()
{
}

void DesktopView::resizeEvent(QResizeEvent* event)
{
    zoomToFit();
    QGraphicsView::resizeEvent(event);
}

void DesktopView::zoomToFit()
{
    DesktopScene *scn = qobject_cast<DesktopScene *>( scene() );

    if (scn)
    {
        scn->setSceneRect( scn->getSceneRect() );
        fitInView(scene()->sceneRect(), Qt::KeepAspectRatio);
        scene()->update( scene()->sceneRect() );
        scale(1 / 1.1, 1 / 1.1);
    }
}

void DesktopView::zoomIn()
{
    scale(1.1, 1.1);
    emit zoomChanged();
}

void DesktopView::zoomOut()
{
    scale(1 / 1.1, 1 / 1.1);
    emit zoomChanged();
}
