
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#ifndef _NETWORKMANAGER_H_
#define _NETWORKMANAGER_H_

#include <QtCore/QObject>
#include <QtNetwork/QHostAddress>

class QTimer;
class TcpClient;
class TcpServer;
class NetworkMessage;

class NetworkManager :
    public QObject
{
    Q_OBJECT

public:

    enum NetworkMode
    {
        MODE_UNCONNECTED = 0,
        MODE_SERVER,
        MODE_CLIENT
    };

    NetworkManager(QObject *parent = 0);
    virtual ~NetworkManager();

    int getPort() const {return m_port; }
    int getMode() const {return m_mode; }
    const QString & getServerName() const {return m_serverName; }
    QAbstractSocket::SocketState getState() const;
    QString getStateAsString() const;
    int getClientNb() const;

signals:

    void serverAlive();
    void serverDown();
    void connectionChanged();
    void newMessage(const QString&);
    void newMessage(const NetworkMessage &);

public slots:

    void start();
    void stop();
    void setServerName(const QString &name);
    void setPort(int port);
    void setMode(int mode);

    void sendNewMessage(const QString &);
    void sendNewMessage(const NetworkMessage &);
    void saveSettings();

private slots:

    void clientConnectionChanged(QAbstractSocket::SocketState);
    void serverConnectionChanged(quint8);

private:

    void updateConfig();
    void loadSettings();
    void initialize();

    int m_port;
    QString m_serverName;
    int m_mode;
    QString m_currentMessage;
    bool m_initialized;
    TcpClient *m_client;
    TcpServer *m_server;
    bool m_started;
};

#endif // _NETWORKMANAGER_H_
