
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtGui/QPainter>

#include "StarDelegate.h"
#include "StarEditor.h"
#include "StarRating.h"

StarDelegate::StarDelegate(QWidget *parent) : QStyledItemDelegate(parent),
    m_column(0)
{
}

void StarDelegate::setStarColumn(int val)
{
    m_column = val;
}

void StarDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                         const QModelIndex &index) const
{
    if (index.column() == m_column)
    {
        StarRating starRating( index.data().toInt() );

        if (option.state & QStyle::State_Selected)
            painter->fillRect( option.rect, option.palette.highlight() );
        else
        {
            QColor c = index.data(Qt::BackgroundRole).value<QColor>();
            if ( c.isValid() )
                painter->fillRect( option.rect,  c);
        }

        starRating.paint(painter, option.rect, option.palette,
                         StarRating::ReadOnly);
    }
    else
    {
        QStyledItemDelegate::paint(painter, option, index);
    }
}

QSize StarDelegate::sizeHint(const QStyleOptionViewItem &option,
                             const QModelIndex &         index) const
{
    if (index.column() == m_column)
    {
        return StarRating( index.data().toInt() ).sizeHint();
    }
    else
    {
        return QStyledItemDelegate::sizeHint(option, index);
    }
}

QWidget *StarDelegate::createEditor(QWidget *                   parent,
                                    const QStyleOptionViewItem &option,
                                    const QModelIndex &         index) const

{
    if (index.column() == m_column)
    {
        StarEditor *editor = new StarEditor(parent);
        connect( editor, SIGNAL( editingFinished() ),
                 this, SLOT( commitAndCloseEditor() ) );
        return editor;
    }
    else
    {
        return QStyledItemDelegate::createEditor(parent, option, index);
    }
}

void StarDelegate::setEditorData(QWidget *          editor,
                                 const QModelIndex &index) const
{
    if (index.column() == m_column)
    {
        StarRating starRating( index.data().toInt() );
        StarEditor *starEditor = qobject_cast<StarEditor *>(editor);
        starEditor->setStarRating(starRating);
    }
    else
    {
        QStyledItemDelegate::setEditorData(editor, index);
    }
}

void StarDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                const QModelIndex &index) const
{
    if (index.column() == m_column)
    {
        StarEditor *starEditor = qobject_cast<StarEditor *>(editor);
        model->setData( index, starEditor->starRating().starCount() );
    }
    else
    {
        QStyledItemDelegate::setModelData(editor, model, index);
    }
}

void StarDelegate::commitAndCloseEditor()
{
    StarEditor *editor = qobject_cast<StarEditor *>( sender() );
    emit commitData(editor);
    emit closeEditor(editor);
}
